function[M,P,Q,pointsWA,pointsNA]=EIVbiasadjust(mt,St,n,T,nbpoints)
%inputs:
%mt is the n_chi x 1 vector of estimated measures.
%St is the n_chi x 1 vector of estimated asymptotic variance of the estimated measures.
%both mt and St are computed for the estimated values that pass the fund selection
%rule (number of observations and condition number (Eq. 8)).
%n is the total number of units (before the selection rule).
%T is the total number of observations.
%nbpoints is the total number of values for the grid of
%points m (such as 1,000 or 2,000).

%outputs: 
%M is the vector of bias-adjusted (moments, standard deviation, skewness,
%kurtosis).
%P is the vector of bias-adjusted proportions (negative and positive).
%Q is the vector of bias-adjusted quantiles (5% and 95%).
%pointsWA is the nbpoints x 2 matrix, which contains the grid of points m and the bias-adjusted density value. 
%pointsNA is the nbpoints x 2 matrix, which contains the grid of points m and the bias-unadjusted density value. 


%coefficients reference model
muAlpha = mean(mt);
stdAlpha = std(mt);
st=log(St);
mus = mean(st);
stds = std(st);
rho = corr(mt,st);
rho2=rho^2;
seuil=(6-2*sqrt(6))/stds^2;
if rho2>seuil,
   rho=sqrt(seuil)-0.1*sqrt(seuil);
end

%computation of the optimal bandwidth (n large, T small)
cOne = 3/(4*stdAlpha^5);
const=exp(mus + 0.5*stds^2*(1 - (rho^2)/2));
cTwo = cOne*((rho^4*stds^4)/12 - rho^2*stds^2 + 1)*const;
h = cTwo^(-1/3)*(n/T)^(-1/3);

%computation of the standard density (No bias adjustment (NA))
MaxObs=max(mt)-h;
MinObs=min(mt)+h;
step=(MaxObs-MinObs)/(nbpoints-1);
pointsNA=[[MinObs:step:MaxObs];zeros(1,nbpoints)];
pointsNA=pointsNA';
for i=1:nbpoints,
    d=(mt-pointsNA(i,1))/h;
    pointsNA(i,2)=mean(exp(-0.5*(d.^2))/(h*(2*pi)^0.5));
end

%computation of the two bias terms
%smoothing bias
d=(pointsNA(:,1) - muAlpha)/stdAlpha;
phi=(1/sqrt(2*pi))*exp(-0.5*d.^2);
phiadj=(1/stdAlpha)*phi;
bracket=0.5*h^2*(1/stdAlpha^2)*((d.^2)-1);
pointsbias(:,1) = bracket.*phiadj;

%EIV bias
d=(pointsNA(:,1) - muAlpha-rho*stdAlpha*stds)/stdAlpha;
phi=(1/sqrt(2*pi))*exp(-0.5*d.^2);
phiadj=(1/stdAlpha)*phi;
p=exp(mus+0.5*stds^2);
bracket=(1/(2*T))*p*(1/stdAlpha^2)*((d.^2)-1);
pointsbias(:,2) = bracket.*phiadj;

%computation of the bias-adjusted density
pointsbiasTOT = pointsbias(:,1)+pointsbias(:,2);
pointsBM=pointsNA(:,2)-pointsbiasTOT;
pointsWA=[pointsNA(:,1),pointsBM];

%we put the negative values equal to zero
pointsWA(:,2)=max(pointsWA(:,2),0);

%we check that it sums to one
density=trapz(pointsWA(:,1),pointsWA(:,2));
pointsWA(:,2)=pointsWA(:,2)/density;
density=trapz(pointsNA(:,1),pointsNA(:,2));
pointsNA(:,2)=pointsNA(:,2)/density;
%pointsWA=pointsNA;
        
%computation of the moments
M=zeros(4,1);
%mean
M(1,1)=mean(mt);
%volatility
a1=trapz(pointsWA(:,1),pointsWA(:,2).*pointsWA(:,1));
d2=(pointsWA(:,1)-a1).^2;
a2=trapz(pointsWA(:,1),pointsWA(:,2).*d2);
M(2,1)=sqrt(a2);
%skewness
d3=(pointsWA(:,1)-a1).^3;
a3=trapz(pointsWA(:,1),pointsWA(:,2).*d3);
M(3,1)=a3/((a2)^(3/2));
%kurtosis
d4=(pointsWA(:,1)-a1).^4;
a4=trapz(pointsWA(:,1),pointsWA(:,2).*d4);
M(4,1)=a4/((a2)^(2));

%computation of the proportions
P=zeros(2,1);
indic=find(pointsWA(:,1)<0);
P(1,1)=trapz(pointsWA(indic,1),pointsWA(indic,2));
P(2,1)=1-P(1,1);

%computation of the quantiles
Q=zeros(2,1);
nbpoints=size(pointsWA,1);
probtot=zeros(nbpoints,1);
for i=2:nbpoints,
    probtot(i,1)=trapz(pointsWA(1:i,1),pointsWA(1:i,2));
end
z=abs(probtot-ones(nbpoints,1)*0.05);
minz=min(z);
indic=find(z==minz);
Q(1,1)=pointsWA(indic(1,1),1);
z=abs(probtot-ones(nbpoints,1)*0.95);
minz=min(z);
indic=find(z==minz);
Q(2,1)=pointsWA(indic(1,1),1);
